
from sanic import Request
from sqlalchemy.sql import Select
from co6co.utils import log
from sqlalchemy.ext.asyncio import AsyncSession
from .baseCache import BaseCache
from co6co_db_ext.db_utils import DbCallable, db_tools
from ..model.pos.other import sysConfigPO


async def get_upload_path(request: Request) -> str:
    """
    获取上传路径
    数据库未配置使用 /upload
    """
    key = "SYS_CONFIG_UPLOAD_PATH"
    cache = ConfigCache(request)
    path = cache.getConfig(key)
    if path == None:
        await cache.queryConfig(key)
        path = cache.getConfig(key)
    if path == None:
        path = "/upload"
        log.warn("未配置上传路径:key:{},使用默认值:{}".format(key, path))
    return path


class ConfigCache(BaseCache):

    def __init__(self, request: Request) -> None:
        super().__init__(request)

    @property
    def configKeyPrefix(self):
        return 'ConfigKey'

    def getKey(self, code: str):
        """
        获取Key
        """
        return "{}_{}".format(self.configKeyPrefix, code)

    async def queryConfig(self, code: str) -> str | None:
        """
        查询当前用户的所拥有的角色
        结果放置在cache中
        """
        callable = DbCallable(ConfigCache.session(self.request))

        async def exe(session) -> str | None:
            select = (
                Select(sysConfigPO.name, sysConfigPO.code,
                       sysConfigPO.value, sysConfigPO.remark)
                .filter(sysConfigPO.code.__eq__(code))
            )
            data: dict | None = await db_tools.execForMappings(session, select, queryOne=True)
            result = None
            if data == None:
                log.warn("query {} config is NULL".format(code))
            else:
                result = data.get("value")
                self.setConfig(code, result)
            return result

        return await callable(exe)

    def setConfig(self, code: str, value: str):
        if code != None:
            self.setCache(self.getKey(code), value)

    def getConfig(self, code: str) -> str | None:
        if code != None:
            return self.getCache(self.getKey(code))

    def clear(self, code: str) -> str | None:
        if code != None:
            return self.remove(self.getKey(code))




from ..pos.right import UserPO,UserGroupPO
from sqlalchemy .orm.attributes import InstrumentedAttribute
from typing import Tuple
from co6co_db_ext.db_filter import absFilterItems
from co6co.utils import log
from sqlalchemy import func, or_, and_, Select


class user_filter(absFilterItems):
    """
        用户表过滤器
    """
    name: str = None
    userGroupId: int = None
    state:int=None

    def __init__(self, userName=None, userGroupId: int = None):
        super().__init__(UserPO)
        self.name = userName
        self.userGroupId = userGroupId
        self.listSelectFields=[UserPO.id,UserPO.userGroupId,UserPO.state,UserPO.createTime, UserPO.userName]
         

    def filter(self) -> list:
        """
        过滤条件
        """
        filters_arr = []
        if self.checkFieldValue(self.userGroupId):
            filters_arr.append(UserPO.userGroupId.__eq__(self.userGroupId))
        if self.checkFieldValue(self.state):
            filters_arr.append(UserPO.state.__eq__(self.state))
        if self.checkFieldValue(self.name):
            filters_arr.append(UserPO.userName.like(f"%{self.name}%"))
        return filters_arr

    def create_List_select(self):
        select=(
				Select(UserPO.id,UserPO.userGroupId,UserPO.state,UserPO.createTime, UserPO.userName,UserGroupPO.name.label("groupName"),UserGroupPO.id.label("groupId"))
                .join(UserGroupPO,isouter=True,onclause=UserPO.userGroupId==UserGroupPO.id)
				.filter(and_(*self.filter()))  
		)
        return select

    def getDefaultOrderBy(self) -> Tuple[InstrumentedAttribute]:
        """
        默认排序
        """
        return (UserPO.createTime.desc(),)




from co6co_web_db.view_model import BaseMethodView

from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.services.jwt_service import createToken, setCurrentUser
from co6co.utils import log

from co6co_web_db.view_model import get_one
from ..model.pos.right import UserPO, RolePO, UserRolePO, AccountPO
from .aop.login_log import loginLog
from ..services import getSecret, generatePageToken


class login_view(BaseMethodView):
    routePath = "/login"

    @loginLog
    async def post(self, request: Request):
        """
        登录
        """
        where = UserPO()
        where.__dict__.update(request.json)
        select = Select(UserPO).filter(UserPO.userName.__eq__(where.userName))
        user: UserPO = await get_one(request, select)
        verifyCode = request.json.get("verifyCode", "")
        _, sessionDict = self.get_Session(request)
        memCode = sessionDict.pop("verifyCode")
        # // todo 为什么 应用 重启后 session 还在
        if verifyCode == "" or memCode != verifyCode:
            return JSON_util.response(Result.fail(message="验证码不能为空!"))
        if user != None:
            if user.password == user.encrypt(where.password):
                tokenData = await generatePageToken(getSecret(request), user, userOpenId=user.userGroupId)
                # 让日志能获得用户信息
                await setCurrentUser(request, user.to_jwt_dict())
                return JSON_util.response(Result.success(data=tokenData, message="登录成功"))
            else:
                return JSON_util.response(Result.fail(message="登录用户名或者密码不正确!"))
        else:
            log.warn(f"未找到用户名[{where.userName}]。")
            return JSON_util.response(Result.fail(message="登录用户名或者密码不正确!"))
